﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using HelixToolkit.Wpf;
using Plane.FormationCreator.Formation;
using Plane.Geography;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D;

namespace Plane.FormationCreator.ViewModels
{
    public class View3DViewModel : ViewModelBase
    {
        private FlightTaskManager _flightTaskManager;

        public View3DViewModel(FlightTaskManager flightTaskManager)
        {
            modelGroup = new Model3DGroup();
            planeGroup = new Model3DGroup();
            waypointGroup = new Model3DGroup();
            modelGroup.Children.Add(planeGroup);
            modelGroup.Children.Add(waypointGroup);
            this.Model = modelGroup;
            _flightTaskManager = flightTaskManager;
            _flightTaskManager.PropertyChanged += new PropertyChangedEventHandler(flightTaskPropertyChanged);
        }

        private void flightTaskPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case nameof(FlightTaskManager.SelectedTaskIndex):
                        SelectTask();
                    break;
                default:
                    break;
            }
        }

        private void SelectTask()
        {
            if (_flightTaskManager.SelectedTaskIndex > 0)
            {
                //modelGroup.Children.Clear();
                if (waypointGroup == null) waypointGroup = new Model3DGroup();
                waypointGroup.Children.Clear();
               

                var meshBuilderwaypoint = new MeshBuilder(false, false);
                meshBuilderwaypoint.AddSphere(new Point3D(0, 0, 0), 0.3);
                var meshwaypoint = meshBuilderwaypoint.ToMesh(true);
                var greenMaterial = MaterialHelper.CreateMaterial(Color.FromRgb(0,255,0));

                foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.Tasks[_flightTaskManager.SelectedTaskIndex].SingleCopterInfos)
                {
                
                    double x = GeographyUtils.CalcDistance(_flightTaskManager.OriginLng, 0, 0, info.TargetLng, 0, 0);
                    if (_flightTaskManager.OriginLng > info.TargetLng) x = -x;
                    double y = GeographyUtils.CalcDistance(0, _flightTaskManager.OriginLat, 0, 0, info.TargetLat, 0);
                    if (_flightTaskManager.OriginLat > info.TargetLat) y = -y;

                    waypointGroup.Children.Add(new GeometryModel3D
                    {
                        Geometry = meshwaypoint,
                        Transform = new TranslateTransform3D(x, y, info.TargetAlt),
                        Material = greenMaterial,
                        BackMaterial = greenMaterial
                    });
                }
            }
        }

        public Model3D Model { get; set; }
        public Model3DGroup modelGroup { get; set; }

        public Model3DGroup planeGroup { get; set; }
        public Model3DGroup waypointGroup { get; set; }

    }
}
